import { system, world, EquipmentSlot, EntityEquippableComponent } from "@minecraft/server";


export const getObjective = (id) => world.scoreboard.getObjective(id) ?? world.scoreboard.addObjective(id, id);

export function getEquipment(player) {
  const tick = system.currentTick;
  if (player._equip?.tick === tick) return player._equip.comp;
  const comp = player.getComponent(EntityEquippableComponent.componentId);
  player._equip = { tick, comp };
  return comp;
}

export function getArmor(player, ignoreCache = false) {
  const equip = getEquipment(player);
  if (!equip) return {};
  const tick = system.currentTick;
  if (!ignoreCache && player._armor?.tick === tick) {
    return player._armor.items;
  }
  const items = {
    head: equip.getEquipment(EquipmentSlot.Head),
    chest: equip.getEquipment(EquipmentSlot.Chest),
    legs: equip.getEquipment(EquipmentSlot.Legs),
    feet: equip.getEquipment(EquipmentSlot.Feet),
  };
  player._armor = { tick, items };
  return items;
}

let playersCache;
export function getPlayers() {
  const tick = system.currentTick;
  if (playersCache?.tick === tick) return playersCache.players;
  const players = world.getPlayers();
  playersCache = { tick, players };
  return players;
}